@@ -6,6 +6,7 @@ from typing import Union, Iterable
 from dnn_cool.converters import Values, Converters
 from dnn_cool.runner import DnnCoolSupervisedRunner
 from dnn_cool.task_flow import TaskFlow
+from dnn_cool.verbosity import log, StatsRegistry, Verbosity
 
 
 def not_found_error_message(col, df):
@@ -20,38 +21,40 @@ def assert_col_in_df(col, df):
             assert col_s in df, not_found_error_message(col_s, df)
 
 
-def create_values(df, output_col, converters):
+def create_values(df, output_col, converters, perform_conversion):
+    log(f'Creating values from column "{output_col}" in dataframe ...')
     values_type = converters.type.guess(df, output_col)
-    values = converters.values.to_values(df, output_col, values_type)
+    values = converters.values.to_values(df, output_col, values_type, perform_conversion)
     return values
 
 
-def create_leaf_task(df, col, converters):
-    values = create_values(df, col, converters)
+def create_leaf_task(df, col, converters, perform_conversion):
+    values = create_values(df, col, converters, perform_conversion)
     task = converters.task.to_task(col, values.types[0], values.values[0])
     return task
 
 
-def create_leaf_tasks(df, col, converters):
+def create_leaf_tasks(df, col, converters, perform_conversion):
     if isinstance(col, str):
-        return [create_leaf_task(df, col, converters)]
+        return [create_leaf_task(df, col, converters, perform_conversion)]
 
     res = []
     for col_s in col:
-        res.append(create_leaf_task(df, col_s, converters))
+        res.append(create_leaf_task(df, col_s, converters, perform_conversion))
     return res
 
 
-def read_inputs(df, input_col, converters):
+def read_inputs(df, input_col, converters, perform_conversion):
+    log(f'Reading inputs from dataframe...')
     if isinstance(input_col, str):
-        values = create_values(df, input_col, converters)
+        values = create_values(df, input_col, converters, perform_conversion)
         return values
 
     keys = []
     values = []
     types = []
     for col_s in input_col:
-        vals = create_values(df, col_s, converters)
+        vals = create_values(df, col_s, converters, perform_conversion)
         keys.extend(vals.keys)
         values.extend(vals.values)
         types.extend(vals.types)
@@ -87,23 +90,30 @@ class Project:
                  input_col: Union[str, Iterable[str]],
                  output_col: Union[str, Iterable[str]],
                  project_dir: Union[str, Path],
-                 converters: Converters):
+                 converters: Converters,
+                 verbosity: Verbosity = Verbosity.SILENT):
         self.df = df
-        assert_col_in_df(input_col, df)
-        assert_col_in_df(output_col, df)
+        perform_conversion = df is not None
+        self.perform_conversion = perform_conversion
+        if perform_conversion:
+            assert self.df is not None
+            assert_col_in_df(input_col, df)
+            assert_col_in_df(output_col, df)
 
         self.project_dir = Path(project_dir)
         self.project_dir.mkdir(exist_ok=True)
 
-        converters_file = self.project_dir / 'converters.pkl'
+        self.stats_registry = StatsRegistry(self.project_dir / 'stats_registry.pkl', verbosity)
+        converters_directory = self.project_dir / 'converters'
         if converters is None:
             converters = Converters()
         self.converters = converters
-        if converters_file.exists():
-            self.converters.load_state_dict(torch.load(converters_file))
+        self.converters.connect_to_stats_registry(self.stats_registry)
+        if converters_directory.exists() and perform_conversion:
+            self.converters.load_state_from_directory(converters_directory)
 
-        self.inputs = read_inputs(df, input_col, converters)
-        self.leaf_tasks = create_leaf_tasks(df, output_col, converters)
+        self.inputs = read_inputs(df, input_col, converters, perform_conversion)
+        self.leaf_tasks = create_leaf_tasks(df, output_col, converters, perform_conversion)
 
         self.flow_tasks = []
 
@@ -113,8 +123,8 @@ class Project:
 
         for i in range(len(self.inputs.keys)):
             self.converters.tensorboard_converters.col_to_type_mapping[self.inputs.keys[i]] = self.inputs.types[i]
-        if not converters_file.exists():
-            torch.save(self.converters.state_dict(), converters_file, pickle_protocol=4)
+        if not converters_directory.exists():
+            self.converters.dump_state_to_directory(converters_directory)
 
     def add_task_flow(self, task_flow: TaskFlow):
         self.flow_tasks.append(task_flow)
@@ -155,4 +165,5 @@ class Project:
                                        early_stop=early_stop,
                                        balance_dataparallel_memory=balance_dataparallel_memory,
                                        runner_name=runner_name,
-                                       train_test_val_indices=train_test_val_indices)
+                                       train_test_val_indices=train_test_val_indices,
+                                       perform_conversion=self.perform_conversion)
