@@ -1,10 +1,12 @@
 import os
 from dataclasses import dataclass, field
-from typing import Iterable, Optional, Callable, Tuple, Sequence
+from typing import Iterable, Optional, Callable, Tuple, Sequence, List
 
 import torch
+import numpy as np
 from torch import nn
 from torch.utils.data import Dataset
+from treelib import Tree, Node
 
 from dnn_cool.activations import CompositeActivation
 from dnn_cool.datasets import FlowDataset, LeafTaskDataset
@@ -17,8 +19,8 @@ from dnn_cool.metrics import TorchMetric, get_default_binary_metrics, \
     get_default_bounded_regression_metrics, get_default_classification_metrics, \
     get_default_multilabel_classification_metrics
 from dnn_cool.missing_values import positive_values
-from dnn_cool.modules import SigmoidAndMSELoss, Identity, TaskFlowModule
-from dnn_cool.treelib import TreeExplainer
+from dnn_cool.modules import SigmoidAndMSELoss, Identity, TaskFlowModule, IModuleOutput
+from dnn_cool.treelib import TreeExplainer, default_leaf_tree_explainer
 
 
 class ITask:
@@ -135,6 +137,9 @@ class Task(ITask):
     def get_dataset(self, **kwargs):
         return LeafTaskDataset(self.inputs, self.labels)
 
+    def get_treelib_explainer(self) -> Callable:
+        return default_leaf_tree_explainer
+
 
 @dataclass()
 class BinaryHardcodedTask(Task):
@@ -208,6 +213,25 @@ class ClassificationTask(Task):
     module: nn.Module = Identity()
     metrics: Sequence[Tuple[str, TorchMetric]] = field(default_factory=get_default_classification_metrics)
 
+    class_names: Optional = None
+    top_k: Optional[int] = 5
+
+    def get_treelib_explainer(self) -> Callable:
+        def classification_explainer(task_name: str,
+                                     decoded: torch.Tensor,
+                                     activated: torch.Tensor,
+                                     logits: torch.Tensor,
+                                     node_identifier: str) -> Tuple[Tree, Node]:
+            tree = Tree()
+            start_node = tree.create_node(task_name, node_identifier)
+            for i, idx in enumerate(decoded[:self.top_k]):
+                name = idx if self.class_names is None else self.class_names[idx]
+                description = f'{i}: {name} | activated: {activated[idx]:.4f}, logits: {logits[idx]:.4f}'
+                tree.create_node(description, f'{node_identifier}.{idx}', parent=start_node)
+            return tree, start_node
+
+        return classification_explainer
+
 
 @dataclass()
 class MultilabelClassificationTask(Task):
@@ -227,6 +251,26 @@ class MultilabelClassificationTask(Task):
     module: nn.Module = Identity()
     metrics: Sequence[Tuple[str, TorchMetric]] = field(default_factory=get_default_multilabel_classification_metrics)
 
+    class_names: Optional = None
+
+    def get_treelib_explainer(self) -> Callable:
+        def explainer(task_name: str,
+                      decoded: np.ndarray,
+                      activated: np.ndarray,
+                      logits: np.ndarray,
+                      node_identifier: str) -> Tuple[Tree, Node]:
+            tree = Tree()
+            start_node = tree.create_node(task_name, node_identifier)
+            for i, val in enumerate(decoded):
+                name = i if self.class_names is None else self.class_names[i]
+                description = f'{i}: {name} | decoded: {decoded[i]}, ' \
+                              f'activated: {activated[i]:.4f}, ' \
+                              f'logits: {logits[i]:.4f}'
+                tree.create_node(description, f'{node_identifier}.{i}', parent=start_node)
+            return tree, start_node
+
+        return explainer
+
 
 class TaskFlow(ITask):
 
@@ -271,7 +315,7 @@ class TaskFlow(ITask):
         return all_metrics
 
     def get_treelib_explainer(self):
-        return TreeExplainer(self)
+        return TreeExplainer(self.get_name(), self.get_flow_func(), self.tasks)
 
     def get_labels(self, *args, **kwargs):
         all_labels = []
